﻿using System;
using System.Linq;
using System.Text.RegularExpressions;

namespace Txt2Bib.Records
{
    public interface IBib {
        public string Convert(string[] entryLines);
        public string ToString();
    };

    public abstract record ItemType
    {
        protected static bool IsDoi(string url)
        {
            return url.Contains("doi");
        }
        protected static string Rearrange(string a)
        {
                a = a.Trim();
                var s = a.Split(' ');
                return $"{s[1]} {s[0]}";
        }
    }

    public record class Article : ItemType, IBib
    {
        public string Type { get; init; } = "article";
        public string[] Author { get; set; } = { "Gianni e Pinotto" };
        public string Title { get; set; } = "";
        public string Journal { get; set; } = "";
        public ushort Year { get; set; } = 1950;
        public string Volume { get; set; } = "";
        public string? Issue { get; set; } = null;
        public ushort FirstPage { get; set; } = 1;
        public ushort LastPage { get; set; } = 1;
        public string Doi { get; set; } = "";
        public string Url { get; set; } = "";

        public string Convert(string[] entryLines)
        {
            var checkVol = (string v) =>
                v.Split(',').Length == 2 ? v.Split(',')[0].Trim() : v.Trim();

            Author = entryLines[1].Split(',').Select(a => Rearrange(a)).ToArray();
            Year = ushort.Parse(entryLines[2]);
            Title = entryLines[3];
            Journal = entryLines[4];
            Volume = entryLines[5] != string.Empty ? checkVol(entryLines[5]) : Volume;
            Issue = entryLines[5].Split(',').Length == 2 ?
               entryLines[5].Split(',')[1].Trim() : Issue;
            try
            {
                FirstPage = ushort.Parse(entryLines[6].Split('-')[0]);
                LastPage = ushort.Parse(entryLines[6].Split('-')[1].TrimEnd('.'));
            }
            catch (Exception)
            {
                throw;
            }

            if (entryLines.Length > 7 )
            {
                if (IsDoi(entryLines[7])) Doi = entryLines[7];
                else Url = entryLines[7];
            }

            return ToString();
        }

        public override string ToString()
        {
            return $"@{Type}{{{Author[0][..5].Replace(". ", "_")}_{Year},\n" +
                $"\tauthor = \"{string.Join(" and ", Author)}\",\n" +
                $"\ttitle = {{{Title}}},\n" +
                $"\tjournal = \"{Journal}\",\n" +
                $"\tyear = \"{Year}\",\n" +
                $"\tvolume = \"{Volume}\",\n" +
                $"\tnumber = \"{Issue}\",\n" +
                $"\tpages = \"{FirstPage}--{LastPage}\",\n" +
                $"\tdoi = \"{Doi}\",\n" +
                $"\turl = \"{Url}\",\n" +
            "}\n";
        }
    }

    public record class Book : ItemType, IBib
    {
        public string Type { get; init; } = "book";
        public string[] Author { get; set; } = Array.Empty<string>();
        public string[] Editor { get; set; } = Array.Empty<string>();
        public string Title { get; set; } = "";
        public ushort Year { get; set; } = 1950;
        public string Publisher { get; set; } = "";
        public string Place { get; set; } = "";
        public string Url { get; set; } = "";
        public string Doi { get; set; } = "";

        public string Convert(string[] entryLines)
        {
            var auths = entryLines[1];
            if (auths.Contains("eds"))
            {
                Editor = auths.Replace("(eds.)", "").Split(',').Select(a => Rearrange(a)).ToArray();
            }
            else
            {
                Author = auths.Split(',').Select(a => Rearrange(a)).ToArray();
            }
            Year = ushort.Parse(entryLines[2]);
            Title = entryLines[3];
            Place = entryLines[4];
            Publisher = entryLines[5];

            if (entryLines.Length > 6 )
            {
                if (IsDoi(entryLines[6])) Doi = entryLines[6];
                else Url = entryLines[6];
            }

            return ToString();
        }

        public override string ToString()
        {
            string label;
            string authString;

            if (Author.Length != 0)
            {
                label = Author[0][..5].Replace(". ", "");
                authString = $"\tauthor = \"{string.Join(" and ", Author)}\",\n";
            }
            else
            {
                label = Editor[0][..5].Replace(". ", "");
                authString = $"\teditor = \"{string.Join(" and ", Editor)}\",\n";
            }

            return $"@{Type}{{{label}_{Year},\n" +
                authString +
                $"\ttitle = {{{Title}}},\n" +
                $"\tpublisher = \"{Publisher}\",\n" +
                $"\taddress = \"{Place}\",\n" +
                $"\tyear = \"{Year}\",\n" +
                $"\turl = \"{Url}\",\n" +
                $"\tdoi = \"{Doi}\",\n" +
                "}\n";
        }
    }

    public record class Proceedings : ItemType, IBib
    {
        public string Type { get; init; } = "inproceedings";
        public string[] Author { get; set; } = Array.Empty<string>();
        public string Title { get; set; } = "";
        public string BookTitle { get; set; } = "";
        public ushort Year { get; set; } = 1950;
        public string Publisher { get; set; } = "";
        public string Address { get; set; } = "";
        public ushort FirstPage { get; set; } = 1;
        public ushort LastPage { get; set; } = 1;
        public string Url { get; set; } = "";
        public string Doi { get; set; } = "";

        public string Convert(string[] entryLines)
        {
            Author = entryLines[1].Split(',').Select(a => Rearrange(a)).ToArray();
            Year = ushort.Parse(entryLines[2]);
            Title = entryLines[3];
            BookTitle = entryLines[4];
            Address = entryLines[5] != String.Empty ? entryLines[5] : Address;
            Publisher = entryLines[6];
            try
            {
                FirstPage = ushort.Parse(entryLines[7].Split('-')[0]);
                LastPage = ushort.Parse(entryLines[7].Split('-')[1].TrimEnd('.'));
            }
            catch (Exception)
            {
                throw;
            }

            if (entryLines.Length > 8 )
            {
                if (IsDoi(entryLines[8])) Doi = entryLines[8];
                else Url = entryLines[8];
            }

            return ToString();
        }

        public override string ToString()
        {
            string label = Author[0][..5].Replace(". ", "");
            string authString = $"\tauthor = \"{string.Join(" and ", Author)}\",\n";

            return $"@{Type}{{{label}_{Year},\n" +
                authString +
                $"\ttitle = {{{Title}}},\n" +
                $"\tbooktitle = {{{BookTitle}}},\n" +
                $"\tpublisher = \"{Publisher}\",\n" +
                $"\taddress = \"{Address}\",\n" +
                $"\tyear = \"{Year}\",\n" +
                $"\tpages = \"{FirstPage}--{LastPage}\",\n" +
                $"\turl = \"{Url}\",\n" +
                $"\tdoi = \"{Doi}\",\n" +
                "}\n";
        }
    }

    public record class Chapter : ItemType, IBib
    {
        public string Type { get; init; } = "incollection";
        public string[] Author { get; set; } = Array.Empty<string>();
        public string[] Editor { get; set; } = Array.Empty<string>();
        public string Title { get; set; } = "";
        public string BookTitle { get; set; } = "";
        public ushort Year { get; set; } = 1950;
        public string Publisher { get; set; } = "";
        public string Address { get; set; } = "";
        public ushort FirstPage { get; set; } = 1;
        public ushort LastPage { get; set; } = 1;
        public string Url { get; set; } = "";
        public string Doi { get; set; } = "";

        public string Convert(string[] entryLines)
        {
            Author = entryLines[1].Split(',').Select(a => Rearrange(a)).ToArray();
            Year = ushort.Parse(entryLines[2]);
            Title = entryLines[3];
            Editor = entryLines[4].Replace("(eds.)", "").Split(',').Select(a => Rearrange(a)).ToArray();
            BookTitle = entryLines[5];
            Address = entryLines[6] != String.Empty ? entryLines[6] : Address;
            Publisher = entryLines[7];
            try
            {
                FirstPage = ushort.Parse(entryLines[8].Split('-')[0]);
                LastPage = ushort.Parse(entryLines[8].Split('-')[1].TrimEnd('.'));
            }
            catch (Exception)
            {
                throw;
            }

            if (entryLines.Length > 9)
            {
                if (IsDoi(entryLines[9])) Doi = entryLines[9];
                else Url = entryLines[9];
            }

            return ToString();
        }

        public override string ToString()
        {
            string label = Author[0][..5].Replace(". ", "");
            string authString = $"\tauthor = \"{string.Join(" and ", Author)}\",\n";
            string edsString = $"\teditor = \"{string.Join(" and ", Editor)}\",\n";

            return $"@{Type}{{{label}_{Year},\n" +
                authString +
                edsString +
                $"\ttitle = {{{Title}}},\n" +
                $"\tbooktitle = {{{BookTitle}}},\n" +
                $"\tpublisher = \"{Publisher}\",\n" +
                $"\taddress = \"{Address}\",\n" +
                $"\tyear = \"{Year}\",\n" +
                $"\tpages = \"{FirstPage}--{LastPage}\",\n" +
                $"\turl = \"{Url}\",\n" +
                $"\tdoi = \"{Doi}\",\n" +
                "}\n";
        }
    }
}